"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const discord_js_1 = require("discord.js");
const AutoRegister_1 = __importDefault(require("./function/AutoRegister"));
const Common_1 = __importDefault(require("./function/Common"));
const LoadCommands_1 = __importDefault(require("./function/LoadCommands"));
class App {
    options;
    common = new Common_1.default(this);
    commands = new discord_js_1.Collection();
    groups = new discord_js_1.Collection();
    constructor(options) {
        this.options = options;
        let now = Date.now();
        (0, AutoRegister_1.default)(this).then(() => {
            console.log(`\n[P] AutoRegister took ${Date.now() - now}ms`);
            this.client.login(this.options.token).then(() => {
                if (this.commands.size == 0)
                    return;
                now = Date.now();
                (0, LoadCommands_1.default)(this).then(() => {
                    console.log(`\n[P] Loading Commands took ${Date.now() - now}ms`);
                    if (this.options.commandsRegistry.interactionHandler) {
                        this.onCommand();
                    }
                });
            });
        });
    }
    // Getters
    get client() {
        return this.options.client;
    }
    get Common() {
        return this.common;
    }
    get guilds() {
        return this.options.commandsRegistry.allGuilds ?
            this.client.guilds.cache.map(r => {
                return r.id;
            }) :
            this.options.commandsRegistry.guilds;
    }
    onCommand() {
        this.client.on("interactionCreate", interaction => {
            if (!interaction.isCommand())
                return;
            const name = interaction.commandName;
            const command = this.commands.get(name);
            if (!(interaction.channel instanceof discord_js_1.TextChannel))
                return;
            if (interaction.guild == null)
                return;
            if (!command) {
                const group = this.groups.get(name);
                if (!group)
                    return;
                const sub = interaction.options.getSubcommand(true);
                if (!sub)
                    return;
                const cmd = group.__c.find(r => r.name == sub);
                if (!cmd)
                    return;
                cmd.execute({
                    channel: interaction.channel,
                    guild: interaction.guild,
                    interaction: interaction,
                    member: interaction.member
                });
            }
            try {
                if (!command)
                    return; // lame ts error lol
                command.execute({
                    channel: interaction.channel,
                    guild: interaction.guild,
                    interaction: interaction,
                    member: interaction.member
                });
            }
            catch (error) {
                console.log("[B] Error while trying to run command " + name + " :");
                console.log(error);
            }
        });
    }
}
exports.default = App;
